# (C) Copyright IBM Corp. 2020.

from enum import Enum


class QueryType(Enum):
    simpleQuery=1
    complexQuery=2
    randomQuery=3


class SentenceQueryBase:
    def __init__(self, is_ordered, window_size,query_type=QueryType.simpleQuery):
        self.is_ordered = is_ordered
        self.slop = self.adjust_slot(window_size)
        self.query_type=query_type

    def adjust_slot(self, slop):
        return int(slop)

    def get_query_elements(self):
        raise NotImplementedError("Please Implement this method")

    def get_sentence_query(self):
        return {"type":self.query_type.name, "slop":self.slop,"inOrder": self.is_ordered, "clauses" : self.get_query_elements()}

    @staticmethod
    def _get_element(value, field):
        return {"value": value, "sentenceSearchableField": ["sentenceSearchableField", field]}

    @staticmethod
    def get_concept_element(value):
        return SentenceQueryBase._get_element(value, "Concept")

    @staticmethod
    def get_normalized_element(value):
        return SentenceQueryBase._get_element(value, "Normalized")

    @staticmethod
    def get_type_element(value):

        return SentenceQueryBase._get_element(value, "Type")
#///////
#// one list of sets where each set is a dict with type and field
#    // concept
#    // normalize
#    // type
#    // surface


class SimpleQuery(SentenceQueryBase):
    def __init__(self, is_ordered = True, window_size = 50):
        SentenceQueryBase.__init__(self, is_ordered, window_size, QueryType.simpleQuery)
        self.and_elements = list()

    def add_concept_element(self, values):
        # make sure values are written as wikipedia titles, with underscores replacing spaces
        renamed_values = [value.replace(" ", "_") for value in values]
        self.and_elements.append([self.get_concept_element(value) for value in renamed_values])

    def add_normalized_element(self, values):
        self.and_elements.append([self.get_normalized_element(value) for value in values])

    def add_type_element(self, values):
        # replace Sentiment and Causality by full names:  Sentiment -> WP2_sentiment
        # and  Causality -> WP2_action_verb_phrase , (or) WP3_connectors_2
        renamed_values = list()
        for value in values:
            if value == 'Sentiment':
                renamed_values.append('WP2_sentiment')
            elif value == 'Causality':
                renamed_values.append('WP2_action_verb_phrase')
                renamed_values.append('WP3_connectors_2')
            else: renamed_values.append(value)
        self.and_elements.append([self.get_type_element(value) for value in renamed_values])

    def get_query_elements(self):
        return self.and_elements


class RandomQuery(SentenceQueryBase):

    def __init__(self):
        SentenceQueryBase.__init__(self, False,1,QueryType.randomQuery)

    def get_query_elements(self):
        return []


class ConceptNumberQuery(SentenceQueryBase):
    def __init__(self, is_ordered, window_size, concept="Video_game_controversies"):
        SentenceQueryBase.__init__(self, is_ordered, window_size)
        self.concept = concept

    def get_query_elements(self):
        and_elements = list()
        #and_elements.append([self.get_type_element("Year")])
        #and_elements.append([self.get_normalized_element(x) for x in ["bans", "ban", "banned", "banning"]])
        and_elements.append([self.get_concept_element(self.concept)])
        and_elements.append([self.get_type_element(x) for x in ["Number", "Percent", "Money"]])
        return and_elements



# (C) Copyright IBM Corp. 2020.
import datetime
import logging
from debater_python_api.api.clients.abstract_client import AbstractClient

endpoint = '/score/'

class ArgumentQualityClient(AbstractClient):
    def __init__(self, apikey):
        AbstractClient.__init__(self, apikey)
        self.host = 'https://arg-quality.debater.res.ibm.com'

    def run(self, sentence_topic_dicts, timeout=60):
        time_stamp_start = datetime.datetime.now().timestamp()
        for sentence_topic_dict in sentence_topic_dicts:
            if (len(sentence_topic_dict['sentence']) == 0 or len(sentence_topic_dict['topic']) == 0):
                raise RuntimeError('empty input argument in pair {}'.format(sentence_topic_dict))
        pairs = [[dict_item['sentence'], dict_item['topic']] for dict_item in sentence_topic_dicts]
        scores = self.run_in_batch(list_name='sentence_topic_pairs', list=pairs, other_payload={}, endpoint=endpoint, timeout=timeout)
        time_stamp_end = datetime.datetime.now().timestamp()
        logging.info('argument_quality_client.run = {}ms.'.format(1000*(time_stamp_end - time_stamp_start)))

        return scores



from enum import Enum

from debater_python_api.api.debater_api import DebaterApi


def production_host(original_host):
    return original_host


def production_backdoor_host(original_host):
    return original_host.replace('debater.res.ibm.com', 'ris2-debater-event.us-east.containers.appdomain.cloud')


def staging_host(original_host):
    return original_host.replace('https://', 'https://staging-')

def test_host(original_host):
    return original_host.replace('https://', 'https://test-')


def experiments_host(original_host):
    return original_host.replace('https://keypoint-matching-', 'https://kp-')


def production2_host(original_host):
    return original_host.replace('https://keypoint-matching-', 'https://experiments-kp-')


def local_host(_):
    return 'http://localhost:9004'


def staging_backdoor_host(original_host):
    if __name__ == '__main__':
        return original_host.replace('debater.res.ibm.com', 'ris2-debater-event.us-east.containers.appdomain.cloud').\
            replace('https://', 'https://staging-')


def default_host(original_host):
    return original_host.replace('https://', 'https://default-')


def default_backdoor_host(original_host):
    if __name__ == '__main__':
        return original_host.replace('debater.res.ibm.com', 'ris2-debater-event.us-east.containers.appdomain.cloud').\
            replace('https://', 'https://default-')


def decorator(func, host_modifier):
    def inner_function(*args, **kwargs):
        client = func(*args, **kwargs)
        client.host = host_modifier(client.host)
        return client

    return inner_function


class Domains(Enum):
    production = (1, production_host)
    production2 = (2, production2_host)
    staging = (3, staging_host)
    test = (4, test_host)
    production_backdoor = (5, production_backdoor_host)
    staging_backdoor = (6, staging_backdoor_host)
    default = (7, default_host)
    default_backdoor = (8, default_backdoor_host)
    local = (9, local_host)
    experiments = (10, experiments_host)

class DebaterApiWithAdjustedUrl(DebaterApi):

    def __init__(self, apikey, domain):
        super().__init__(apikey)

        self.url_modifier = domain.value[1]
        for attr in dir(self):
            if str(attr).startswith('get_') and str(attr).endswith('client'):
                setattr(self, attr, decorator(getattr(self, attr), self.url_modifier))


def main():
    debater_api = DebaterApiWithAdjustedUrl('PUT_YOUR_API_KEY_HERE', Domains.test)
    client = debater_api.get_evidence_detection_client()
    print('host {}'.format(client.host))
    client = debater_api.get_keypoints_client()
    print('host {}'.format(client.host))


if __name__ == '__main__':
    main()



# (C) Copyright IBM Corp. 2020.
import datetime
import logging

from debater_python_api.api.clients.abstract_client import AbstractClient

endpoint='/score/'

class ProConClient(AbstractClient):

    def __init__(self, apikey):
        AbstractClient.__init__(self, apikey)
        self.host = 'https://pro-con.debater.res.ibm.com'

    def run(self, sentence_topic_dicts):
        time_stamp_start = datetime.datetime.now().timestamp()
        for i, dict_ in enumerate(sentence_topic_dicts):
            if len(dict_['sentence']) == 0:
                raise RuntimeError('empty input sentence among the inputs at index {}'.format(i))
            if len(dict_['topic']) == 0:
                raise RuntimeError('empty imput topic among the inputs at index {}'.format(i))
        pairs = [[dict_['sentence'], dict_['topic']] for dict_ in sentence_topic_dicts]

        scores = self.run_in_batch(list_name='sentence_topic_pairs', list=pairs, other_payload={}, endpoint=endpoint)

        scores = [dict['pro'] - dict['con'] for dict in scores]

        time_stamp_end = datetime.datetime.now().timestamp()
        logging.info('pro_con_client.run = {}ms.'.format(1000*(time_stamp_end - time_stamp_start)))

        return scores




# (C) Copyright IBM Corp. 2020.

import pandas as pd
import os
from debater_python_api.api.debater_api import DebaterApi
from spacy.lang.en import English
from prettytable import PrettyTable


api_key = os.environ['API_KEY']
debater_api = DebaterApi(api_key)

####start##part1
data_file = os.path.join("..", "resources", "dataset_austin.csv")
input_frame = pd.read_csv(data_file, encoding='utf8')
nlp = English()
nlp.add_pipe(nlp.create_pipe('sentencizer'))
input_frame['sentences'] = input_frame.apply(lambda x: [sent.string.strip() for sent in nlp(x['Comment']).sents],axis=1)
sentence_based_frame = pd.DataFrame(input_frame['sentences'].tolist(), index=input_frame['Comment']).stack()
sentence_based_frame = sentence_based_frame.reset_index([0, 'Comment']).rename(columns={0: 'sentences'})
input_frame = pd.merge(sentence_based_frame, input_frame[['Comment', 'Council District', 'Year']], on='Comment', how='left')
####end##part1

####start##part2
keypoints_client = debater_api.get_keypoints_client()
arg_quality_client = debater_api.get_argument_quality_client()

years = input_frame['Year'].unique().tolist()
sentences_per_year = {}
ids_per_year = {}
domain = 'kp_based_survey_example'
for year in years:
    year_frame = input_frame[input_frame['Year'] == year]
    sentences = year_frame['sentences'].tolist()
    # use argument quality service to select top 1000 sentences
    arg_quality_scores = arg_quality_client.run([{'sentence': sentence, 'topic': 'Austin'} for sentence in sentences])
    sentences = [sentence for _, sentence in sorted(zip(arg_quality_scores, sentences), reverse=True)]
    sentences_per_year[year] = sentences[:1000]
####end##part2

####start##part3
year = 2016
ids_per_year[year] = ['{}_{}'.format(year, i) for i in range(len(sentences_per_year[year]))]
keypoints_client.upload_comments(domain=domain, comments_ids=ids_per_year[year], comments_texts=sentences_per_year[year])
####end##part3

####start##part4

# run kp analysis for 2016
keypoints_client.wait_till_all_comments_are_processed(domain)
future = keypoints_client.start_kp_analysis_job(domain=domain, comments_ids=ids_per_year[2016])
keypoint_matchings = future.get_result(high_verbosity=True, polling_timout_secs=5)  # waits till result is available
####end##part4

####start##part5
# print results in a table
kp_2016_table = PrettyTable()
kp_2016_table.field_names = ['#', 'key point', 'size', 'example']
j = 0
for kp in keypoint_matchings['keypoint_matchings']:
    if kp['keypoint'] == 'none':    # skip cluster of all unmatched sentences
        continue
    j += 1
    kp_2016_table.add_row([j, kp['keypoint'], len(kp['matching']), ''])
    num_of_examples = 0
    for i, matching in enumerate(kp['matching']):  # print 3 matched (short enough for readability) sentences
        if i == 0:  # The first example it the kp itself.
            continue
        sentence = matching['sentence_text']
        if len(sentence) < 65:
            kp_2016_table.add_row(['', '', '', matching['sentence_text']])
            num_of_examples += 1
        if num_of_examples >= 2:  # show two examples at table
            break
print('2016 key points:')
print(kp_2016_table)
####end##part5

####start##part6
# run kp analysis for 2017, use the same key points from 2016
# In order to reuse the 2016 key points, we pass 2016 job id as key_points_by_job_id parameter in start_kp_analysis_job.
year = 2017
ids_per_year[year] = ['{}_{}'.format(year, i) for i in range(len(sentences_per_year[year]))]
keypoints_client.upload_comments(domain=domain,
                                 comments_ids=ids_per_year[year],
                                 comments_texts=sentences_per_year[year])
future = keypoints_client.start_kp_analysis_job(domain=domain, comments_ids=ids_per_year[year],
                                                key_points_by_job_id=future.get_job_id())
keypoint_matchings = future.get_result(high_verbosity=True, polling_timout_secs=5)  # waits until results are available

kp_2017_table = PrettyTable()
kp_2017_table.field_names = ['#', 'key point', 'size', 'example']
j = 0
for kp in keypoint_matchings['keypoint_matchings']:
    if kp['keypoint'] == 'none':    # skip cluster of all unmatched sentences
        continue
    j += 1
    kp_2017_table.add_row([j, kp['keypoint'], len(kp['matching']), ''])
    num_of_examples = 0
    for i, matching in enumerate(kp['matching']):  # print 3 matched (short enough for readability) sentences
        if i == 0:  # The first example it the kp itself.
            continue
        sentence = matching['sentence_text']
        if len(sentence) < 65:
            kp_2017_table.add_row(['', '', '', matching['sentence_text']])
            num_of_examples += 1
        if num_of_examples >= 2:  # show two examples at table
            break
print('2017 key points:')
print(kp_2017_table)
####end##part6

####start##part7
keypoints_client.delete_domain_cannot_be_undone(domain)
####end##part7




import datetime
import logging
import time
import traceback
from debater_python_api.api.clients.abstract_client import AbstractClient

class KpPairsInferClient(AbstractClient):
    def __init__(self, apikey, host):
        AbstractClient.__init__(self, apikey)
        self.host = host

    def _post(self, endpoint, body, use_cache=True, retries=10, timeout=60):
        logging.info('client calls service (post): ' + self.host+endpoint)
        updates_to_headers = {}
        if not use_cache:
            updates_to_headers['cache-control'] = 'no-cache'

        return self.do_run(endpoint=endpoint, payload=body, updates_to_headers=updates_to_headers, timeout=timeout, retries=retries)

    # argument_kp_pairs: list of lists with two strings, the first is an argument and the second is a kp. the returned scores are not symmetric
    def get_pairs_inference_scores(self, argument_kp_pairs, batch_size, use_cache=True, retries=10, timeout=60):
        try:
            start_time = datetime.datetime.now()
            scores = []
            batch_data_list = [argument_kp_pairs[x:x + batch_size] for x in range(0, len(argument_kp_pairs), batch_size)]
            for batch in batch_data_list:
                start_time_batch = datetime.datetime.now()
                scores.extend(self._post('/score/', {'pairs': batch}, use_cache,  retries=retries, timeout=timeout))
                logging.info("inferred %d out of %d pairs, batch took: %s" % (len(scores), len(argument_kp_pairs), str(datetime.datetime.now() - start_time_batch)))
            logging.info("inferred %d batches, num_scores: %d,  took: %s" % (len(batch_data_list), len(scores), str(datetime.datetime.now() - start_time)))
            return scores
        except Exception as e:
            track = traceback.format_exc()
            logging.info(track)
            raise Exception('failed to retrieve pairs scores', e)


# (C) Copyright IBM Corp. 2020.
import datetime
import logging
from debater_python_api.api.clients.abstract_client import AbstractClient

endpoint = '/TermWikifier/v2/annotate/'

class TermWikifierClient(AbstractClient):
    def __init__(self, apikey):
        AbstractClient.__init__(self, apikey)
        self.host = 'https://tw.debater.res.ibm.com'

    def run(self, texts):
        time_stamp_start = datetime.datetime.now().timestamp()
        texts_to_annotate = [{'text': text, 'contextTitles': []} for text in texts]
        payload = {
            'config': '',
            'contextTitles': [],
            'contextText': ''
        }
        sentences = self.run_in_batch(list_name='textsToAnnotate', list=texts_to_annotate, other_payload=payload,
                                 list_from_json_getter=lambda x:x['annotations'], endpoint=endpoint)
        term_wikifier_result = []
        for sentence in sentences:
            sentence_results = [{
                'span' : (annotation['spanStart'], annotation['spanEnd']),
                'cleanText' : annotation['cleanText'],
                'concept' : {'title' : annotation['title'] ,
                             'inlinks' : int(annotation['inlinks'])}
            } for annotation in sentence if annotation['title'] is not None]
            term_wikifier_result.append(sentence_results)

        time_stamp_end = datetime.datetime.now().timestamp()
        logging.info('term_wikifier_client.run = {}ms.'.format(1000*(time_stamp_end - time_stamp_start)))

        return term_wikifier_result



# (C) Copyright IBM Corp. 2020.

import os
import matplotlib.pyplot as plt, numpy as np, pandas as pd, os
from collections import namedtuple
from debater_python_api.api.debater_api import DebaterApi
from prettytable import PrettyTable, ALL
from sklearn.cluster import SpectralClustering
from spacy.lang.en import English

api_key = os.environ['API_KEY']
debater_api = DebaterApi(api_key)

def get_sentences_based_frame(comments_based_frame):
    nlp = English()
    nlp.add_pipe(nlp.create_pipe('sentencizer'))
    comments_based_frame['sentences'] = comments_based_frame.apply(lambda x: [sent.string.strip() for sent in nlp(x['Comment']).sents],axis=1)
    sentence_based_frame = pd.DataFrame(comments_based_frame['sentences'].tolist(), index=comments_based_frame['Comment']).stack()
    sentence_based_frame = sentence_based_frame.reset_index([0, 'Comment']).rename(columns={0: 'sentences'})
    return pd.merge(sentence_based_frame, comments_based_frame[['Comment', 'Council District', 'Year']], on='Comment', how='left')

def populate_similarity_matrix(num_themes,relation_scores):
    theme_similarity_matrix = np.zeros([num_themes, num_themes])
    index = 0
    for i in range(num_themes):
        theme_similarity_matrix[i, i] = 1
        for j in range(i + 1, num_themes):
            relation_score = relation_scores[index]
            theme_similarity_matrix[i, j] = relation_score
            theme_similarity_matrix[j, i] = relation_score
            index += 1
    return theme_similarity_matrix

def get_theme_clusters(theme_similarity_matrix, unique_themes):
    theme_clusters = []
    num_theme_clusters = round(num_themes/3)
    theme_clustering = SpectralClustering(num_theme_clusters,random_state=0).fit_predict(theme_similarity_matrix)
    for i in range(num_theme_clusters):
        themes_cluster = [unique_themes[j] for j in range(num_themes) if theme_clustering[j]== i]
        theme_clusters.append(themes_cluster)
    avg_cluster_size = sum([len(cluster) for cluster in theme_clusters]) / len(theme_clusters)
    theme_clusters_without_outliers = [cluster for cluster in theme_clusters if len(cluster) < (2 * avg_cluster_size)]
    outliers = [cluster for cluster in theme_clusters if cluster not in theme_clusters_without_outliers]
    flattened_outliers = [[theme] for cluster in outliers for theme in cluster]
    return theme_clusters_without_outliers + flattened_outliers

def get_representative_themes(theme_clusters, clusters_and_themes_frame):
    theme_represetative = namedtuple('theme_representative', ['representative_theme', 'clustered_themes', 'themes_count'])
    theme_clusters_print_table = PrettyTable(hrules=ALL)
    theme_clusters_print_table.field_names = ['Representing Theme', 'Clustered Themes']
    representative_themes = []
    for theme_cluster in theme_clusters:
        theme_cluster_counts = {theme: clusters_and_themes_frame['main_theme'].tolist().count(theme) for theme in theme_cluster}
        cluster_themes_sorted_by_frequency = sorted(theme_cluster_counts, key=theme_cluster_counts.get, reverse=True)
        representative_theme = cluster_themes_sorted_by_frequency[0]
        representative_themes.append(theme_represetative(representative_theme,theme_cluster,sum(theme_cluster_counts.values())))
    representative_themes.sort(key=lambda x: x.themes_count,reverse=True)
    [theme_clusters_print_table.add_row([theme.representative_theme,theme.clustered_themes]) for theme in representative_themes[:5]]
    print(theme_clusters_print_table)
    return [theme.representative_theme for theme in representative_themes]

def create_graph_and_get_top_themes(frame,series,is_stacked_graph,graph_title):
    supporters_per_rep_theme = frame[representative_themes].sum(axis=0, skipna=True).to_dict()
    top_themes = sorted(supporters_per_rep_theme,reverse=True, key=supporters_per_rep_theme.get)[:10]
    table = pd.pivot_table(data=frame[[series]+top_themes], index=[series], aggfunc=sum, margins=True)
    table = table.transpose().sort_values(by='All').drop('All',axis=1)
    table.plot.barh(stacked=is_stacked_graph)
    plt.title(graph_title)
    plt.legend(title='Districts')
    plt.xlabel('num_sentences')
    plt.tight_layout()
    plt.show()
    return top_themes

####start##part1
data_file = os.path.join("..","resources","dataset_austin.csv")
input_frame = pd.read_csv(data_file, encoding='utf8')
input_frame = get_sentences_based_frame(input_frame)
####end##part1


####start##part2
clustering_client = debater_api.get_clustering_client()
clustering_client.set_seed(0)
theme_extraction_client = debater_api.get_theme_extraction_client()
districts = input_frame['Council District'].unique().tolist()
years = input_frame['Year'].unique().tolist()
clusters_and_themes_info = []
for year in years:
    year_frame = input_frame[input_frame['Year'] == year]
    for district in districts:
        year_n_district_frame = year_frame[year_frame['Council District']==district]
        sentences = year_n_district_frame['sentences'].tolist()
        clusters = clustering_client.run(sentences=sentences, num_of_clusters=12)
        extracted_themes = theme_extraction_client.run(clusters=clusters, topic="We should support the City of Austin", dominant_concept='City of Austin')
        for cluster_index in range(len(clusters)):
            returned_themes = [theme['theme'] for theme in extracted_themes[cluster_index]]
            main_theme = returned_themes[0] if len(returned_themes) > 0 else 'No Theme'
            cluster_elements = clusters[cluster_index]
            cluster_size = len(cluster_elements)
            clusters_and_themes_info.append([year,district,main_theme,cluster_size,cluster_elements])
clusters_and_themes_frame = pd.DataFrame(data=clusters_and_themes_info,
                                         columns=['year','district','main_theme','cluster_size','cluster_elements'])

####end##part2

####start##part3
unique_themes = clusters_and_themes_frame['main_theme'].unique().tolist()
num_themes = len(unique_themes)
relation_pairs = []
for i in range(num_themes):
    theme_i = unique_themes[i]
    for j in range(i+1,num_themes):
        theme_j = unique_themes[j]
        relation_pairs.append([theme_i,theme_j])
term_relater_client = debater_api.get_term_relater_client()
relation_scores = term_relater_client.run(relation_pairs)
theme_similarity_matrix = populate_similarity_matrix(num_themes,relation_scores)
theme_clusters = get_theme_clusters(theme_similarity_matrix, unique_themes)
representative_themes = get_representative_themes(theme_clusters, clusters_and_themes_frame)
####end##part3

####start##part4
term_wikifier_client = debater_api.get_term_wikifier_client()
term_annotations_for_sentences = term_wikifier_client.run(input_frame['sentences'].tolist())
wiki_titles_for_sentences = [[annotation['concept']['title'] for annotation in annotation_array] for annotation_array in term_annotations_for_sentences]
sentences_to_wiki_titles = dict(zip(input_frame['sentences'].tolist(), wiki_titles_for_sentences))
input_frame['sentence_wiki_titles'] = input_frame['sentences'].map(sentences_to_wiki_titles)
####end##part4

####start##part5
def is_supporter(wiki_titles, theme_similarity_to_wiki_titles):
    sentence_similarity_scores = [theme_similarity_to_wiki_titles[title] for title in wiki_titles]
    if len(wiki_titles)>0 and max(sentence_similarity_scores)>0.95:
            return True
    return False
unique_wiki_titles = list(set([title for wiki_titles_for_sentence in wiki_titles_for_sentences for title in wiki_titles_for_sentence]))
for theme in representative_themes:
    relation_pairs_for_theme = [[theme, wiki_title] for wiki_title in unique_wiki_titles]
    theme_relation_scores = term_relater_client.run(relation_pairs_for_theme)
    similarity_of_theme_to_wiki_titles = {unique_wiki_titles[i]: theme_relation_scores[i]
                                          for i in range(len(unique_wiki_titles))}
    input_frame[theme] = input_frame.apply(
        lambda sentence: is_supporter(sentence['sentence_wiki_titles'],
                                      similarity_of_theme_to_wiki_titles),axis=1)
####end##part5

####start##part6
input_2016_frame = input_frame[input_frame['Year']==2016]
top_2016_themes = create_graph_and_get_top_themes(frame= input_2016_frame,
                                                  series='Council District',
                                                  is_stacked_graph= True,
                                                  graph_title='2016 top themes over districts')
####end##part6

####start##part7
argument_quality_client = debater_api.get_argument_quality_client()
claim_detection_client = debater_api.get_claim_detection_client()
scored_sentence = namedtuple('scored_sentence',['sentence','argumentative_score','quality_score'])
for theme in top_2016_themes[:3]:
    sample_texts_table = PrettyTable()
    sample_texts_table.field_names = [theme.upper() + ' sample texts']
    sub_frame = input_2016_frame[input_2016_frame[theme]]
    theme_sentences = sub_frame['sentences'].tolist()
    theme_sentences = [sentence for sentence in theme_sentences if len(sentence) in range(70, 150)]
    sentence_topic_dicts = [{'sentence': sentence, 'topic': 'We should support ' + theme} for sentence in theme_sentences]
    quality_scores = argument_quality_client.run(sentence_topic_dicts)
    argumentative_scores = claim_detection_client.run(sentence_topic_dicts)
    scored_sentences = [
        scored_sentence(sentence=theme_sentences[i],
                        argumentative_score=argumentative_scores[i],
                        quality_score=quality_scores[i])
        for i in range(len(theme_sentences))]
    sample_sentences = [sentence for sentence in scored_sentences if sentence.argumentative_score > 0.5]
    sample_sentences = sorted(sample_sentences, key=lambda x:x.quality_score, reverse=True)[:3]
    [sample_texts_table.add_row([sample_sentence.sentence]) for sample_sentence in sample_sentences]
    print(sample_texts_table)
    print()
####end##part7

####start##part8
create_graph_and_get_top_themes(frame=input_frame,series='Year', is_stacked_graph= False,graph_title='Yearly trends')
####end##part8


# (C) Copyright IBM Corp. 2020.
import json
import zipfile
from datetime import datetime
import time
import logging
import pickle
from enum import Enum

import requests

from debater_python_api.api.clients.abstract_client import AbstractClient
from debater_python_api.api.clients.response_data.speech_response import ArgumentResponse
from debater_python_api.api.clients.response_data.speech_response import ClusterElementResponse
from debater_python_api.api.clients.response_data.speech_response import ClusterResponse
from debater_python_api.api.clients.response_data.speech_response import SpeechElement
from debater_python_api.api.clients.response_data.speech_response import SpeechResult
from debater_python_api.utils import general_utils


class Polarity(Enum):
    PRO = 1
    CON = -1

end_point_speech_generation = '/api/v1/speech/generate/'
end_point_get_customization = '/api/v1/speech/customization/'
end_point_get_assessment    = '/api/v1/speech/assess/'
end_point_retrieve_zipped_assessment = '/api/v1/speech/assess/getzippedresult/'
end_point_get_status_of_speech_construction    = '/api/v1/speech/status/'
end_point_post_speech_assessment_with_KP_labeled_data = '/api/v1/speech/assess/kpLabeledData/'
end_point_post_filters_assessment = '/api/v1/speech/assess/filters/'

spleeping_time = 10 # seconds

class NarrativeGenerationClient(AbstractClient):
    def __init__(self, apikey):
        AbstractClient.__init__(self, apikey)

        self.host = 'https://speech-construction.debater.res.ibm.com'

    def wait_for_completion_of(self, json):
        headers = general_utils.get_default_request_header(self.apikey)
        now = datetime.now()
        starting_to_wait = now.strftime("%d/%m/%Y %H:%M:%S")
        responseId = json['responseId']

        while True:
            attempts = 0
            while True:
                try:
                    json = requests.get(url=self.host+end_point_get_status_of_speech_construction+responseId, headers=headers, timeout=60).json()
                    break
                except requests.exceptions.ReadTimeout as e:
                    print (e)
                    print ('Got timeout exception, trying again.')
                except Exception as t:  ## all other errors
                    attempts += 1
                    print (t)
                    print ('failed attempt {} to access SC re status of {}'.format(attempts, responseId))
                    if attempts >= 10:
                        raise RuntimeError ('Failed to connect to {} 10 times leaving.'.format(self.host+end_point_get_status_of_speech_construction))

            if json['status'] == 'DONE' or json['status'] == 'ERROR':
                return json

            now = datetime.now()
            dt_string = now.strftime("%d/%m/%Y %H:%M:%S")
            logging.info((dt_string+', SpeechResultResponse={}, motionGenerationid={}, status={}, started to wait at '
                         +starting_to_wait).format(responseId, json['motionGenerationId'], json['status']))
            print((dt_string+', SpeechResultResponse={}, motionGenerationid={}, status={}, started to wait at '
                  +starting_to_wait).format(responseId, json['motionGenerationId'], json['status']))
            time.sleep(spleeping_time)

    def run(self, topic, dc, sentences, pro_con_scores, polarity, customizations=[], sbcFlow='USE_THEMES', motionid='0', approvedAsKeypoints = None, additional_headers={}):
        time_stamp_start = datetime.now().timestamp()
        if len(sentences) != len(pro_con_scores):
            raise RuntimeError('The arguments size ({}) should be equal to pro con scores size ({})'.
                               format(len(sentences), len(pro_con_scores)))
        index_translation = sorted(range(len(sentences)), key=lambda k: sentences[k]) # the order in which sentences show alphabetically sorted
        arguments_records = [{'userId': '',
                              'argumentText': sentences[i],
                              'argumentId': i,
                              'userPolarity': 'PRO',
                              'approvedAsKeypoint' : True if approvedAsKeypoints is None else approvedAsKeypoints[i],
                              'systemPolarity': pro_con_scores[i]} for i in index_translation]

        payload = {'motionGenerationId': motionid,
                   'motionText': topic,
                   'creationTime': '',
                   'arguments': arguments_records,
                   'polarity': polarity.name,
                   'dominantConcept': dc,
                   'eventId': "eventId-e2e",
                   'publish': "false",
                   'customizations': customizations,
                   'sbcFlow' : sbcFlow  # one of: USE_THEMES, USE_KEYPOINTS_CLUSTER_LEVEL, USE_KEYPOINTS_MOTION_LEVEL
                   }

        json = self.do_run(payload, endpoint=end_point_speech_generation, updates_to_headers=additional_headers, timeout=60)

        json = self.wait_for_completion_of(json)

        if json['status'] == 'ERROR':
            # error_message = json['errorMessage']
            # speech_generation_uniq_id=json['speechGenerationUniqId']
            return SpeechResult(arguments=None, clusters=None, paragraphs=None,
                                speech_generation_uniq_id=json['speechGenerationUniqId'],
                                status=json['status'], rows_for_kps_csv = None, rows_for_filtered_elements = None,
                                error_message=json['errorMessage'])
            # raise RuntimeError(f'Generation {speech_generation_uniq_id} of speech for topic {topic} and polarity {polarity.name} failed. Error message is: {error_message}')

        arguments_list = [ArgumentResponse(id=arg['argumentId'], text=arg['argumentText'],
                                                        in_speech=arg['inSpeech'],reason=arg['reason'],
                                                        user_id=arg['userId']) for arg in json['arguments']]
        arguments = {arg.id: arg for arg in arguments_list}
        clusters = [ClusterResponse(theme=cluster['theme'], in_speech=cluster['inSpeech'],
                                    elements=[ClusterElementResponse(representative=arguments[element['sbcArgument']['argumentId']],
                                                                     similar_arguments=[arguments[arg['argumentId']] for arg in
                                                                                        element['similarSbcArguments']
                                                                                        ]
                                                                     ) for element in cluster['representativeArguments']
                                              ])
                    for cluster in json['clusters']]
        paragraphs = []
        for paragraph in json['speechParagraphs']:
            paragraph = [SpeechElement(text=element['elementText'], text_with_tts=element['elementTextWithTts'],
                                         contributing_arguments=[arguments[arg['sbcArgument']['argumentId']]
                                                                 for arg in element['contributingArguments']])
                         for element in paragraph['speechElements']]
            paragraphs.append(paragraph)
        time_stamp_end = datetime.now().timestamp()
        logging.info('narrative_generation_client.run = {}ms.'.format(1000*(time_stamp_end - time_stamp_start)))
        speech_generation_uniq_id = json['speechGenerationUniqId']
        status = json['status']
        rows_for_kps_csv = json['rowsOfKPsCsv']
        rows_for_filtered_elements = json['rowsOfFilteredElementsCSV']

        return SpeechResult(arguments=arguments, clusters=clusters, paragraphs=paragraphs,
                            speech_generation_uniq_id=speech_generation_uniq_id, status=status,
                            rows_for_kps_csv = rows_for_kps_csv,
                            rows_for_filtered_elements = rows_for_filtered_elements)

    def get_customization(self):
        headers = general_utils.get_default_request_header(self.apikey)
        response = requests.get(url=self.host+end_point_get_customization, headers=headers, timeout=60)
        if response.status_code != 200:
            dump = {'url': self.host+end_point_get_customization, 'headers': headers}
            x = datetime.now()
            with open(self.__class__.__name__ + x.strftime("%Y-%m-%d-%H-%M-%S") + '.pkl', 'wb') as f:
                pickle.dump(dump, f, pickle.HIGHEST_PROTOCOL)
            raise ConnectionError(
                "Can't access server. Status code: {} - {}".format(response.status_code, response.reason))
        return response.text


    def get_assessments_for_speechGenerationUniqueIds(self, idlist, path_to_dir_where_zip_to_be_stored):
        headers = general_utils.get_default_request_header(self.apikey)
        num_of_attempts = 0
        while True:
            try:
                response = requests.get(url=self.host+end_point_get_assessment+idlist, headers=headers, timeout=60).json()
                if response['status'] == 'ERROR':
                    raise RuntimeError ('An error occurred while generating speech assessment. Error reads: '+response['errorMessage'])
                if response['status'] == 'DONE':
                    break
                print('assessment is not ready yet, trying again soon')
                time.sleep(10)

            except Exception as e:
                num_of_attempts = 1+num_of_attempts
                print('Fail try {} to get assessment.'.format(num_of_attempts))
                print('Whole Exception printout:')
                print(e)
                print('Endof whole Exception printout.')
                if num_of_attempts >= 3:
                    raise Exception('Failed to get assessment in three attempts, leaving.')
                print(f'Trying again to get assessment for speech {idlist}')

        timestamp = '_' + datetime.now().strftime('%Y-%m-%d-%H-%M-%S')
        assessment_zip = path_to_dir_where_zip_to_be_stored + '/assessment'+timestamp+'.zip'
        headers = general_utils.get_default_request_header(self.apikey)
        response = requests.post(url=self.host+end_point_retrieve_zipped_assessment, json=response, headers=headers, timeout=60)
        print('Downloading zip to '+assessment_zip)
        with open(assessment_zip, 'wb') as zip_file:
            zip_file.write(response.content)

        print('Extracting all files to a sibling directory')
        with zipfile.ZipFile(assessment_zip, 'r') as file:
            # extracting the files using 'extracall' method into a newly created directory, a sibling of the zip file, same name.
            file.extractall(assessment_zip[0:-4])
        print('Done!')

    def get_filters_assessmet (self, labeled_data_records, computed_pro_cons_scores, customizations, path_to_dir_where_zip_to_be_stored):
        headers = general_utils.get_default_request_header(self.apikey)
        request = {'pro_con_scores' : computed_pro_cons_scores,
                   'labeled_data_records' : labeled_data_records,
                   'customizations' : customizations}

        response = requests.post(url=self.host+end_point_post_filters_assessment, json=request, headers=headers, timeout=3600)
        timestamp = '_' + datetime.now().strftime('%Y-%m-%d-%H-%M-%S')
        assessment_zip = path_to_dir_where_zip_to_be_stored + '/filter_assessment'+timestamp+'.zip'
        print('saving zipped report into '+assessment_zip)
        with open(assessment_zip, 'wb') as zip_file:
            zip_file.write(response.content)

        with zipfile.ZipFile(assessment_zip, 'r') as file:
            # printing all the information of archive file contents using 'printdir' method
            # print(file.printdir())

            # extracting the files using 'extracall' method into a newly created directory, a sibling of the zip file, same name.
            print('Extracting all files...')
            file.extractall(assessment_zip[0:-4])
        print('Done!')


    def get_assessment_for_speech_with_KP_Labeled_Data (self, list_of_labeled_data_records, evaluation_threshold,
                                                        speech_generation_unique_id, path_to_dir_where_zip_to_be_stored):
        payload = {'speechGenerationUniqeId': speech_generation_unique_id,
                   'labeledDataRecords' :  list_of_labeled_data_records,
                   'evaluationThreshold' : evaluation_threshold
                   }

        # do not use the usual do_run, because we do not receive a usual json here, but rather - a binary string.
        headers = general_utils.get_default_request_header(self.apikey)
        response = requests.post(url=self.host+end_point_post_speech_assessment_with_KP_labeled_data, json=payload, headers=headers, timeout=3600)
        if response.status_code > 200:
            raise ConnectionError("Error generating report with data")
        print('Downloading zip')
        timestamp = '_' + datetime.now().strftime('%Y-%m-%d-%H-%M-%S')
        assessment_zip = path_to_dir_where_zip_to_be_stored + '/assessment_withLabeledData'+timestamp+'.zip'
        with open(assessment_zip, 'wb') as zip_file:
            zip_file.write(response.content)

        with zipfile.ZipFile(assessment_zip, 'r') as file:
            # printing all the information of archive file contents using 'printdir' method
            # print(file.printdir())

            # extracting the files using 'extracall' method into a newly created directory, a sibling of the zip file, same name.
            print('Extracting all files...')
            file.extractall(assessment_zip[0:-4])
            print('Done!')



# (C) Copyright IBM Corp. 2020.
import datetime
import logging

from debater_python_api.api.clients.abstract_client import AbstractClient

endpoint = '/api/public/clustering'

class ClusteringClient(AbstractClient):
    def __init__(self, apikey):
        AbstractClient.__init__(self, apikey)
        self.host = 'https://clustering.debater.res.ibm.com'
        self.text_preprocessing = 'stemming' # stemming, lemmatization, or wikifier, or stemming_over_lemmatization
        self.embedding_method = 'tf' #tf, ifidf, concepts, glove, or bert_ft_concat
        self.clustering_method = 'sib'  # kmeans, skmeans_euclidean, or sib
        self.clustering_configuration = {}

    def set_text_preprocessing(self, text_preprocessing):
        self.text_preprocessing = text_preprocessing

    def set_embedding_method(self, embedding_method):
        self.embedding_method = embedding_method

    def set_clustering_method(self, clustering_method):
        self.clustering_method = clustering_method

    def set_seed(self, seed):
        self.clustering_configuration["random_state"] = seed

    def run(self, sentences, num_of_clusters):
        time_stamp_start = datetime.datetime.now().timestamp()
        clusters_json = self.sentences_to_clusters_json(sentences, num_of_clusters)

        clusters_list = []
        for cluster_info in clusters_json:
            arguments_info_container = cluster_info['argumentInfoContainers']
            argument_list = [sentences[argument['argument_id']] for argument in arguments_info_container]
            clusters_list.append(argument_list)

        time_stamp_end = datetime.datetime.now().timestamp()
        logging.info('clustering_client.run = {}ms.'.format(1000*(time_stamp_end - time_stamp_start)))
        return clusters_list

    def sentences_to_clusters_json(self, sentences, num_of_clusters):
        if not self.is_configuration_correct():
            raise RuntimeError('Clustering error: configuration not set properly. When choosing Clustering method sib, embedding method must be tf.')

        if len(sentences) == 0:
            raise RuntimeError('empty set of input sentences')
        if (len(sentences) < num_of_clusters):
            raise RuntimeError('The number {} of input sentences is smaller than the number {} of requested clusters.'
                               .format(len(sentences), num_of_clusters))

        if self.text_preprocessing == 'stemming_over_lemmatization' :
            textPreprocessing = ['lemmatization' , 'stemming']
        else: textPreprocessing = [str(self.text_preprocessing)]
        payload = {'text_preprocessing': textPreprocessing,
                   'embedding_method': str(self.embedding_method),
                   'clustering_method': str(self.clustering_method),
                   'num_of_clusters': num_of_clusters,
                   'arguments': sorted(sentences),
                   'clustering_configuration': self.clustering_configuration}
        # just here, as we go to the service itself and get results
        # sort sentences before going to service, and on the returned result, translate the indexes into the sorted list of sentences
        # to indexes into the original, non-sorted list of sentences
        # so that these indeces will be userful to the user who only knows the original, non-sorted list of sentences.
        response_json = self.do_run(payload, endpoint=endpoint)
        sorted_sentences_with_distance = response_json['arguments_id_and_distance_per_cluster']
        index_translation = sorted(range(len(sentences)), key=lambda k: sentences[k])
        for per_cluster_sorted_sentences_with_distance in sorted_sentences_with_distance:
            for arg_index_and_distance in per_cluster_sorted_sentences_with_distance['argumentInfoContainers']:
                arg_index_and_distance['argument_id'] = index_translation[arg_index_and_distance['argument_id']]

        return sorted_sentences_with_distance  #where the sentence-indexes refer to the sentences reshuffeled back to original order.


    def run_with_distances (self, sentences, num_of_clusters):
        time_stamp_start = datetime.datetime.now().timestamp()
        clusters_json = self.sentences_to_clusters_json(sentences, num_of_clusters)
        clusters_with_distances_list = []
        for cluster_info in clusters_json:
            arguments_info_container = cluster_info['argumentInfoContainers']
            argument_and_distances_list = [[sentences[argument['argument_id']], argument["distance"]] for argument in arguments_info_container]
            sorted_argument_and_distances_list = sorted(argument_and_distances_list, key=lambda arg_and_dist: arg_and_dist[1])
            clusters_with_distances_list.append(sorted_argument_and_distances_list)
        time_stamp_end = datetime.datetime.now().timestamp()
        logging.info('clustering_client.run_with_distances = {}ms.'.format(1000*(time_stamp_end - time_stamp_start)))
        return clusters_with_distances_list

    def run_with_full_info (self, sentences, num_of_clusters):
        time_stamp_start = datetime.datetime.now().timestamp()
        clusters_json = self.sentences_to_clusters_json(sentences, num_of_clusters)
        cluster_list = []
        for cluster_info in clusters_json:
            arguments_info_container = cluster_info['argumentInfoContainers']
            argument_and_distances_list = [
                {'text': sentences[argument['argument_id']],
                 'id': argument['argument_id'],
                 'distance': argument['distance']}
                for argument in arguments_info_container
            ]
            sorted_argument_and_distances_list = sorted(argument_and_distances_list, key=lambda arg_and_dist: arg_and_dist['distance'])
            cluster_list.append(sorted_argument_and_distances_list)
        time_stamp_end = datetime.datetime.now().timestamp()
        logging.info('clustering_client.run_with_distances = {}ms.'.format(1000*(time_stamp_end - time_stamp_start)))
        return cluster_list

    def is_configuration_correct(self):
        if self.clustering_method != 'sib':
            return True

        return self.embedding_method == 'tf'



# (C) Copyright IBM Corp. 2020.

import unittest

from debater_python_api.integration_tests.api.clients.DebaterApiWithAdjustedUrl import DebaterApiWithAdjustedUrl
from debater_python_api.integration_tests.api.clients.DebaterApiWithAdjustedUrl import Domains
from debater_python_api.api.sentence_level_index.client.sentence_query_request import SentenceQueryRequest
from debater_python_api.api.sentence_level_index.client.sentence_query_base import RandomQuery
from timeit import default_timer as timer


class ServicesIT(unittest.TestCase):

    def setUp(self):
        self.debater_api = DebaterApiWithAdjustedUrl('PUT_YOUR_API_KEY_HERE', Domains.production)
        self.num_of_sentences = 1000
        self.sentence_length = (10,30)

    def get_sentences(self):
        for i in range(5):
            try:
                searcher = self.debater_api.get_index_searcher_client()

                query = RandomQuery()
                query_request = SentenceQueryRequest(query=query.get_sentence_query(), size=self.num_of_sentences,
                                                     sentenceLength=self.sentence_length)
                return searcher.run(query_request)
            except Exception as e:
                print("Exception in searcher", e)

    def run_service(self, service_runner):
        start_time = timer()
        service_runner()
        run_time = timer() - start_time
        return run_time

    def test_argument_quality_service(self):
        sentences = self.get_sentences()
        motion = 'we should subsidize public transportation'
        time = self.run_service(service_runner=lambda: self.debater_api.get_argument_quality_client().
             run([{'sentence': candidate, 'topic': motion} for candidate in sentences]))
        print('Run argument quality. time={}'.format(time))

    def test_procon_service(self):
        sentences = self.get_sentences()
        motion = 'we should subsidize public transportation'
        time = self.run_service(service_runner=lambda: self.debater_api.get_pro_con_client().
                                run([{'sentence': candidate, 'topic': motion} for candidate in sentences]))
        print('Run procon. time={}'.format(time))

    def test_evidence_service(self):
        sentences = self.get_sentences()
        motion = 'we should subsidize public transportation'
        time = self.run_service(service_runner=lambda: self.debater_api.get_evidence_detection_client().
                               run([{'sentence': candidate, 'topic': motion} for candidate in sentences]))
        print('Run evidence. time={}'.format(time))

    def test_claim_service(self):
        sentences = self.get_sentences()
        motion = 'we should subsidize public transportation'
        time = self.run_service(service_runner=lambda: self.debater_api.get_claim_detection_client().
                               run([{'sentence': candidate, 'topic': motion} for candidate in sentences]))
        print('Run claim. time={}'.format(time))

    def test_claim_boundaries_service(self):
        sentences = self.get_sentences()
        time = self.run_service(service_runner=lambda: self.debater_api.get_claim_boundaries_client().
                                run(sentences=sentences))
        print('Run claim boundaries. time={}'.format(time))

    def test_term_wikifier_service(self):
        sentences = self.get_sentences()
        time = self.run_service(service_runner=lambda: self.debater_api.get_term_wikifier_client().run(sentences))
        print('Run term wikifier. time={}'.format(time))

    def test_clustering_service(self):
        sentences = self.get_sentences()
        time = self.run_service(service_runner=lambda: self.debater_api.get_clustering_client().run(sentences,10))
        print('Run clustering(cluster {} sentences to {} clusters) . time={}'.format(len(sentences), 10, time))

    def test_term_relater(self):
        num_of_pairs_of_concepts = 100
        sentences = self.get_sentences()
        term_wikifier_client = self.debater_api.get_term_wikifier_client()
        mentions = term_wikifier_client.run(sentences)
        concepts = [val['concept']['title'] for sublist in mentions for val in sublist]
        concepts = list(set(concepts))[0:num_of_pairs_of_concepts*2]
        pairs = [[concepts[i], concepts[-i - 1]] for i in range(num_of_pairs_of_concepts)]
        time = self.run_service(service_runner=lambda: self.debater_api.get_term_relater_client().run(pairs))
        print('Run term relater on {} pairs. time={}'.format(num_of_pairs_of_concepts, time))

if __name__ == '__main__':
    unittest.main()



# (C) Copyright IBM Corp. 2020.

import json
import logging
import datetime

from debater_python_api.api.clients.abstract_client import AbstractClient
from debater_python_api.api.sentence_level_index.client.elastic_search_sentence_res import ElasticSentencesSearchRes


class IndexServiceClient(AbstractClient):

    def __init__(self, apikey):
        AbstractClient.__init__(self, apikey)
        self.host = 'https://wiki-index.debater.res.ibm.com'
        self.default_index_name = 'wiki18_v1'
        self.version = '3.0'

    def get_endpoint(self, endpoit_var):
        return '/search/'+ self.default_index_name + '/ln_document/' + endpoit_var + '?version=' + self.version

    def run(self, query, return_full_record=False):
        # this ugly code is here because one of the fields the json object should contain is called "from",
        # however, this cannot be a name of a member in Python :(
        time_stamp_start = datetime.datetime.now().timestamp()
        query_dict = vars(query)
        query_dict["from"] = query_dict["start"]
        query_dict.pop("start")
        logging.info ("customer of api_key {}, submitted a query: {}".format(self.apikey, json.dumps(query_dict)))
        res = self.do_run(query_dict, endpoint=self.get_endpoint('andOfOrs'))
        results = ElasticSentencesSearchRes(res)
        time_stamp_end = datetime.datetime.now().timestamp()
        logging.info('index_searcher_client.run = {}ms.'.format(1000*(time_stamp_end - time_stamp_start)))
        if return_full_record:
            return results.sentences
        return [sentence.cleanSentenceText.strip() for sentence in results.sentences]

    def get_articles(self, article_retrieval_request):
        result = self.run_in_batch(list_name=None, list=article_retrieval_request.articleIds,
                                other_payload=None, endpoint=self.get_endpoint('getArticlesByIds'),
                                list_from_json_getter=lambda x: x['results'])
        return result



# (C) Copyright IBM Corp. 2020.
import math
import sys
import unittest

from hamcrest import assert_that

from debater_python_api.api.clients.keypoints_client import KpAnalysisUtils
from debater_python_api.api.clients.narrative_generation_client import Polarity
from debater_python_api.api.sentence_level_index.client.sentence_query_base import SimpleQuery
from debater_python_api.api.sentence_level_index.client.sentence_query_request import SentenceQueryRequest
from debater_python_api.examples.resources.arguments import arguments_list
from debater_python_api.examples.resources.pro_con_scores import pro_con_scores
from debater_python_api.integration_tests.api.clients.DebaterApiWithAdjustedUrl import DebaterApiWithAdjustedUrl
from debater_python_api.integration_tests.api.clients.DebaterApiWithAdjustedUrl import Domains
from debater_python_api.utils import clusters_refiner


class ServicesIT(unittest.TestCase):

    deployment_env = Domains.production
    # deployment_env = Domains.transformers
    # deployment_env = Domains.test

    def setUp(self):
        self.debater_api = DebaterApiWithAdjustedUrl('PUT_YOUR_API_KEY_HERE', ServicesIT.deployment_env)

    def test_argument_quality_service(self):

        argument_quality_client = self.debater_api.get_argument_quality_client()

        topic = "We should further explore the development of autonomous vehicles"
        sentences = [
            'Cars should only provide assisted driving, not a complete autonomy.',
            'Cars cars cars cars who cares',
            'that he given sun roads sea']

        sentence_topic_dicts = [{'sentence': sentence, 'topic': topic} for sentence in sentences]

        scores = argument_quality_client.run(sentence_topic_dicts)
        assert_that('wrong number of scores returned', len(sentence_topic_dicts) == len(scores))
        for score in scores:
            assert_that(0 <= score <= 1.0)

    def test_claim_detection_service(self):

        claim_detection_client = self.debater_api.get_claim_detection_client()

        topic = 'We should legalize cannabis'
        sentences = ['A cross-party report found good evidence that cannabis treatments can help alleviate the symptoms of chronic pain, multiple sclerosis, nausea and vomiting, particularly in the context of chemotherapy, and anxiety.',
                     'A recent federal study indicates that cannabis is dangerous',
                     'cannabis is dangerous',
                     'The apple tree is green']

        sentence_topic_dicts = [{'sentence' : sentence, 'topic' : topic } for sentence in sentences]

        scores = claim_detection_client.run(sentence_topic_dicts)
        assert_that('wrong number of scores returned', len(sentence_topic_dicts) == len(scores))
        for score in scores:
            assert_that(0 <= score <= 1.0)

    def test_evidence_detection_service(self):

        evidence_detection_client = self.debater_api.get_evidence_detection_client()

        topic = 'We should legalize cannabis'
        sentences = ['A cross-party report found good evidence that cannabis treatments can help alleviate the symptoms of chronic pain, multiple sclerosis, nausea and vomiting, particularly in the context of chemotherapy, and anxiety.',
             'A recent federal study indicates that cannabis is dangerous',
             'cannabis is dangerous',
             'The air in Los Angeles is clear']

        sentence_topic_dicts = [{'sentence' : sentence, 'topic' : topic } for sentence in sentences]

        scores = evidence_detection_client.run(sentence_topic_dicts)
        assert_that('wrong number of scores returned', len(sentence_topic_dicts) == len(scores))
        for score in scores:
            assert_that(0 <= score <= 1.0)

    def test_claim_boundaries_service(self):

        claim_boundaries_client = self.debater_api.get_claim_boundaries_client()

        sentences = ['A cross-party report found good evidence that cannabis treatments can help alleviate the symptoms of chronic pain, multiple sclerosis, nausea and vomiting, particularly in the context of chemotherapy, and anxiety.',
                     'A recent federal study indicates that cannabis is dangerous']

        boundaries_dicts = claim_boundaries_client.run(sentences)

        for i, boundary_dict in enumerate(boundaries_dicts):
            assert_that(boundary_dict['claim'] in sentences[i])

    def test_clustering_service(self):

        clustering_client = self.debater_api.get_clustering_client()

        sentences = ['The cat (Felis catus) is a small carnivorous mammal',
                     'The origin of the domestic dog includes the dogs evolutionary divergence from the wolf',
                     'As of 2017, the domestic cat was the second-most popular pet in the U.S.',
                     'its domestication, and its development into dog types and dog breeds.',
                     'Cats are similar in anatomy to the other felid species',
                     'Dogs are highly variable in height and weight.']

        clusters_list = clustering_client.run(sentences=sentences, num_of_clusters=2)
        assert_that(len(clusters_list) == 2)
        for cluster in clusters_list:
            assert_that(len(cluster) > 0)

        topP = 0.3333
        arguments_and_distances = clustering_client.run_with_distances(sentences, 2)
        assert_that(2 == len(arguments_and_distances))

        splitted_clusters = clusters_refiner.separate_close_from_distant_arguments(arguments_and_distances, topP)
        assert_that(2 == len(splitted_clusters))

        for i in range(0, 2):
            assert_that(math.ceil(len(arguments_and_distances[i]) * topP) == len(splitted_clusters[i][0]))


    def test_index_searcher_service(self):

        index_searcher_client = self.debater_api.get_index_searcher_client()

        query = SimpleQuery(is_ordered=False, window_size=10)
        query.add_concept_element(['Wind_power', 'Wind_farm'])
        query.add_normalized_element(['harnessing', 'batteries'])
        query.add_type_element(['Sentiment'])
        query_request = SentenceQueryRequest(query=query.get_sentence_query(), size=8, sentenceLength=(7, 60))

        sentences = index_searcher_client.run(query_request)
        assert_that(len(sentences) == 8)

    def test_pro_con_service(self):

        pro_con_client = self.debater_api.get_pro_con_client()

        topic = 'Social media is harmful'
        sentences = [
            'Social media disproportionally promotes fake news',
            'Social media is wonderful for human relationship',
            'The air in Los Angeles is clear']

        sentence_topic_dicts = [{'sentence' : sentence, 'topic' : topic } for sentence in sentences]

        scores = pro_con_client.run(sentence_topic_dicts)
        assert_that('wrong number of scores returned', len(sentence_topic_dicts) == len(scores))
        for score in scores:
            assert_that(-1.0 <= score <= 1.0)

    def test_narrative_generation_client(self):

        narrative_generation_client = self.debater_api.get_narrative_generation_client()

        topic = "We should legalize cannabis"
        dominant_concept = "Cannabis"

        speech = narrative_generation_client.run(topic=topic, dc=dominant_concept, sentences=arguments_list,
                                                 pro_con_scores=pro_con_scores, polarity=Polarity.PRO)

        assert_that(len(speech.arguments) > 10)
        assert_that(len(speech.paragraphs) >= 4)
        assert_that(len(speech.clusters) >= 4)

    def test_term_relater_service(self):

        term_relater_client = self.debater_api.get_term_relater_client()

        term_pairs = [['Soldier', 'Army'],
                      ['Pupil', 'Teacher'],
                      ['Ball', 'Tree']]

        scores = term_relater_client.run(term_pairs)
        for score in scores:
            assert_that(0 <= score <= 1.0)


    def test_term_wikifier_service(self):

        term_wikifier_client = self.debater_api.get_term_wikifier_client()

        sentences = [
            'Cars should only provide assisted driving, not complete autonomy',
            'Self driving car technology will be in conflict with vehicles driven by human users']

        annotation_arrays = term_wikifier_client.run(sentences)

        titles = []

        for i, annotation_array in enumerate(annotation_arrays):
            for annotation in annotation_array:
                if annotation['concept']['title'] is not None:
                    titles.append(annotation['concept']['title'])

        assert_that(len(titles) > 0)

    def test_keypoints_service (self):
        comments_texts = [
            "Using cannabis can lead to more dangerous drug use",
            "Prolonged use of cannabis increases the risk of developing psychosis.",
            "Cannabis is a gateway drug to more dangerous drugs",
            "The use of cannabis can be addictive for some people",
            "Legalizing illicit drugs will kill the black market",
            "When cannabis is consumed over a long period of time it has impact on memory and decision making of people.",
            "Early studies suggested cognitive declines associated with marijuana ; these declines persisted long after the period of acute cannabis intoxication."
        ]
        KpAnalysisUtils.init_logger()
        keypoints_client = self.debater_api.get_keypoints_client()

        keypoint_matchings = keypoints_client.run(comments_texts)
        KpAnalysisUtils.print_result(keypoint_matchings)
        assert_that(len(keypoint_matchings['keypoint_matchings']) > 1)
        for keypoint_matching in keypoint_matchings['keypoint_matchings']:
            if keypoint_matching['keypoint'] != 'none':
                for matching in keypoint_matching['matching']:
                    assert_that(0.98 <= matching['score'] <= 1.0)


    def test_theme_extraction_service(self):

        theme_extraction_client = self.debater_api.get_theme_extraction_client()

        dominant_concept = 'Animal'
        topic = 'We should love animals'
        clusters = [
            [
                'The cat (Felis catus) is a small carnivorous mammal',
                'As of 2017, the domestic cat was the second-most popular pet in the U.S.',
                'Cats are similar in anatomy to the other felid species'
            ],
            [
                'The origin of the domestic dog includes the dogs evolutionary divergence from the wolf',
                'its domestication, and its development into dog types and dog breeds.',
                'Dogs are highly variable in height and weight'
            ]
        ]

        theme_extraction_results = theme_extraction_client.run(topic, dominant_concept, clusters)
        assert_that(len(theme_extraction_results[0]) == 1)  # single theme
        assert_that(len(theme_extraction_results[1]) == 0)  # no themes


if __name__ == '__main__':
    if len(sys.argv) > 1:
        deployment_env_str = sys.argv[1]
        if deployment_env_str == 'staging':
            ServicesIT.deployment_env = Domains.staging
        elif deployment_env_str == 'production':
            ServicesIT.deployment_env = Domains.production
        elif deployment_env_str == 'test':
            ServicesIT.deployment_env = Domains.test
        else:
            print('error specifying running environment parameter, options are: test/staging/production')
            exit(1)

    unittest.main(argv=['first-arg-is-ignored'], exit=True)



# (C) Copyright IBM Corp. 2020.

from debater_python_api.api.clients.abstract_client import AbstractClient

endpoint = '/api/public/embedding'

class EmbeddingClient(AbstractClient):
    def __init__(self, apikey):
        AbstractClient.__init__(self, apikey)
        self.host = 'https://clustering.debater.res.ibm.com'
        self.text_preprocessing = [] # stemming, or lemmatization, or stemming_over_lemmatization, or wikifier

    def set_text_preprocessing(self, text_preprocessing):
        if text_preprocessing in {'stemming', 'lemmatization', 'wikifier'}:
            self.text_preprocessing = [str(text_preprocessing)]
        elif text_preprocessing == 'stemming_over_lemmatization':
            self.textPreprocessing = ['lemmatization' , 'stemming']
        else:
            self.text_preprocessing = []

    def set_embedding_method(self, embedding_method):
        self.embedding_method = embedding_method

    def run(self, embedding_method, sentences, mask=''):
        """
        :param embedding_method: one of 'tfidf', 'tf', 'glove'
                                'bert_ft_sent', 'bert_ft_title', 'bert_ft_concat' will be available shortly
        :param sentences: a list of strings to be embedded
        :param mask: a string to ignore
        :return: a list of embeddings

        Default text preprocessing:
        tf/idf: stemming, lemmatization, ignore punctuations, ignore stopwords, apply masking when available
        glove: ignore punctuations, ignore stopwords, apply masking when available
        bert: apply masking when available

        Similarity calculation:
        The BERT models were fine-tuned to capture thematic similarity using the squared Euclidean distance.
        When calculating similarity between BERT embeddings (e1, e2), use L22 distance as follows:
            import numpy as np
            dist = np.linalg.norm(e1 - e2, axis=1)
        When calculating similarity between tf/idf/glove embeddings, use cosine similarity.

        Masking:
        The 'mask' parameter specifies tokens to ignore. The 'tfidf', 'tf', 'glove' methods remove these tokens from
        each string before creating the embedding. BERT concatenates the mask to each string before applying the model.
        """
        if len(sentences) == 0:
            raise RuntimeError('empty set of input sentences')
        payload = {'text_preprocessing': self.text_preprocessing,
                   'embedding_method': str(embedding_method),
                   'embedding_configuration': {'max_df': 1.0, 'min_df': 1, 'motion_text': mask}}
        if embedding_method == 'tfidf':
            payload['arguments'] = sentences
            embeddings_list = self.do_run(payload, endpoint=endpoint)['arguments_embedding']
        else:
            embeddings_list = self.run_in_batch(list_name='arguments', list=sentences, other_payload=payload,
                                                list_from_json_getter=lambda x:x['arguments_embedding'], endpoint=endpoint)
        return embeddings_list



import math

# given a list of clusters, each cluster expressed as a list of ArgumentAndDistance sorted by increasing distance,
# and given p (expressed as a float 0..1)
# return the clusters expressed each as a pair of lists of arguments:
# those that are the top p close to the centroid of the cluster, and the rest of the arguments.

def separate_close_from_distant_arguments(clusters_of_arguments_with_distances, p):
    res = []
    for one_cluster in clusters_of_arguments_with_distances:
        num_of_close_arguments = int(math.ceil(p * len(one_cluster)))
        sorted_args_and_dists = sorted(one_cluster, key=lambda argAndDist: argAndDist[1])
        all_arguments_sorted_by_dists = [arg_and_dist[0] for arg_and_dist in sorted_args_and_dists]
        close_arguments = all_arguments_sorted_by_dists[:num_of_close_arguments]
        distant_arguments = all_arguments_sorted_by_dists[num_of_close_arguments:]
        res.append([close_arguments, distant_arguments])
    return res




# (C) Copyright IBM Corp. 2020.

from debater_python_api.api.debater_api import DebaterApi

debater_api = DebaterApi('PUT_YOUR_API_KEY_HERE')

theme_extraction_client = debater_api.get_theme_extraction_client()

dominant_concept = 'Animal'
topic = 'We should love animals'
clusters = [
    [
        'The cat (Felis catus) is a small carnivorous mammal',
        'As of 2017, the domestic cat was the second-most popular pet in the U.S.',
        'Cats are similar in anatomy to the other felid species'
    ],
    [
        'The origin of the domestic dog includes the dogs evolutionary divergence from the wolf',
        'Domestic dogs have been selectively bred for millennia for various behaviors, sensory capabilities, and physical attributes.',
        'Dogs are highly variable in height and weight'
    ]
]

theme_extraction_results = theme_extraction_client.run(topic, dominant_concept, clusters)
for result in theme_extraction_results:
    print ('Themes for cluster: \n[')
    for theme_dict in result:
        print('  Theme: {}, of pValue = {}'.format(theme_dict['theme'], "{:.4f}".format(theme_dict['pValue'])))
    print (']')
    print()



from debater_python_api.api.debater_api import DebaterApi
from debater_python_api.api.clients.keypoints_client import KpAnalysisUtils

debater_api = DebaterApi('PUT_YOUR_API_KEY_HERE')
keypoints_client = debater_api.get_keypoints_client()

comments_texts = [
    'Cannabis has detrimental effects on cognition and memory, some of which are irreversible.',
    'Cannabis can severely impact memory and productivity in its consumers.',
    'Cannabis harms the memory and learning capabilities of its consumers.',
    'Frequent use can impair cognitive ability.',
    'Cannabis harms memory, which in the long term hurts progress and can hurt people',
    'Frequent marijuana use can seriously affect short-term memory.',
    'Marijuana is very addictive, and therefore very dangerous'
    'Cannabis is addictive and very dangerous for use.',
    'Cannabis can be very harmful and addictive, especially for young people',
    'Cannabis is dangerous and addictive.'
                  ]

KpAnalysisUtils.init_logger()
keypoint_matchings = keypoints_client.run(comments_texts)
KpAnalysisUtils.print_result(keypoint_matchings, print_matches=True)



# (C) Copyright IBM Corp. 2020.

import datetime

from debater_python_api.api.sentence_level_index.client.elastic_search_sentence_res import ElasticSentenceRes


class ElasticArticleRetrievalRes:
    def __init__(self, article_retrieval_res):
        articleFields = article_retrieval_res["articleFields"];

        self.articleLength = articleFields["articleLength"]
        self.author = articleFields["author"]
        self.articleId = articleFields["articleId"]
        self.fullText = articleFields["fullText"]
        self.publisher = articleFields["publisher"]
        self.dateOfPublish = datetime.datetime.fromtimestamp(int(articleFields["dateOfPublish"])/1000)
        self.mediaType = articleFields["mediaType"]
        self.sourceName = articleFields["sourceName"]
        self.title = articleFields["title"]

        self.sentences = [ElasticSentenceRes(sentence,self) for  sentence in article_retrieval_res["sentencesFields"]]
        self.sentences.sort(key=lambda x:x.sentencePosition)


# (C) Copyright IBM Corp. 2020.

import requests
import debater_python_api.examples.example_of_sc_args as sc_example

base_server_url_prefix = 'https://'
base_server_url = '.ris2-debater-event.us-east.containers.appdomain.cloud/'

"""
This function provides an example to how to send a simple request to clustering service
The clustering service receives list of sentences and clustering configuration, and returns a list of clusters,
where each cluster contains a list with it arguments id and the argument distance from the cluster centroid. 

The configuration contains the following fields:
text preprocessing - lemmatization, staming, wikification
embedding method - tfidf, tf, concepts, glove (mean vector), bert
clustering method - the options are: kmeans, skmeans_euclidean (sequential k means with L2 norm), skmeans_cosine )sequential k means with cosine similarity, sib (sequential information bottleneck)
number of clusters - the number of the clusters

"""
def show_clustering_example():
    HEADERS = {'accept-encoding': 'gzip, deflate', 'content-type': 'application/json;charset=UTF-8'}
    uri = 'https://clustering-server.debater-event.us-south.containers.appdomain.cloud:443/api/public/clustering'
    arguments = ['The cat (Felis catus) is a small carnivorous mammal',
                 'The origin of the domestic dog includes the dogs evolutionary divergence from the wolf',
                 'As of 2017, the domestic cat was the second-most popular pet in the U.S.',
                'its domestication, and its development into dog types and dog breeds.',
                 'Cats are similar in anatomy to the other felid species',
                 'Dogs are highly variable in height and weight.']
    payload = {'text_preprocessing': ['stemming'],
               'embedding_method': 'tfidf',
               'clustering_method': 'kmeans',
               'num_of_clusters': 2,
               'arguments':arguments}

    response = requests.post(url=uri, json=payload, headers=HEADERS)

    print(response.json())


"""
This function provides an example to how to send a simple request to quality service
The quality service receives list of pairs where each pair contains an argument and a topic, and returns a list of scores in the range [0, 1],

"""
def show_quality_example():
    HEADERS = {'accept-encoding': 'gzip, deflate', 'content-type': 'application/json;charset=UTF-8'}
    uri = base_server_url_prefix + 'arg-quality' + base_server_url + 'score/'
    payload = {'pairs': [['Cars should only provide assisted driving, not a complete autonomy', 'we should further explore the development of autonomous vehicles'],
                         ['Self driving car technology will be in conflict with vehicles driven by human users', 'we should further explore the development of autonomous vehicles'],
                         ['Finally, regarding Job', 'we should further explore the development of autonomous vehicles']]}

    response = requests.post(url=uri, json=payload, headers=HEADERS)

    print(response.json())


    """
    This function provides an example to how to send a simple request to pro con service
    The pro con service receives list of arguments and a topic, and for each argument, it returns a score in the range [-1,1] 
    that represents the amount of the argument is supporting the topic.

    """
def show_pro_con_example():
    HEADERS = {'accept-encoding': 'gzip, deflate', 'content-type': 'application/json;charset=UTF-8'}
    uri = base_server_url_prefix + 'pro-con' + base_server_url + 'score/'
    discussion_topic = 'Social media is harmful'
    arguments = ['Social media is information power which can be abused if not used with judgement, and Cognitive capabilities can help qualify author, relevance to personal content to help build careers, communities and innovation.',
                 'Social media disproportionally promotes fake news, or at least a somewhat false version of the actual news. Trustworthy media gets less exposure, and people are affected by the lies they encounter through social media.',
                 'Social media promotes isolation and alienation by providing people an easier means to find only those whose opinions they admire or adhere to instead of broadening ones point of view.',
                 'It is',
                 'It is not',
                 '1 2 3 4 5 6 7']
    payload = {'discussion_topic': discussion_topic,
               'arguments': arguments}
    response = requests.post(url=uri, json=payload, headers=HEADERS)
    print(response.json())

    """
    This function provides an example to how to send a simple request to the evidence classifier
    The evidence classifier service receives list, where each element is a topic concatenated to "^^^", concatenated to 
    the sentence. For each element in the list it returns a score in the range [0, 1], which predicts the probability 
    of the sentence to be an evidence for or against the topic. 

    """


def show_evidence_classifier_example():
    HEADERS = {'content-type': 'application/json',
               'charset': 'UTF-8'}
    uri = base_server_url_prefix + 'wp234-motion-evidence-gpu' + base_server_url + 'score/'
    payload = {'pairs': [
        'We should legalize cannabis^^^A cross-party report found good evidence that cannabis treatments can help alleviate the symptoms of chronic pain, multiple sclerosis, nausea and vomiting, particularly in the context of chemotherapy, and anxiety.',
        'We should legalize cannabis^^^cannabis is dangerous']}
    response = requests.post(url=uri, json=payload, headers=HEADERS)
    print(response.json())



    """
     This function provides an example to how to send a simple request to the claim classifier
     The claim classifier service receives list, where each element is a topic concatenated to "^^^", concatenated to 
     the sentence. For each element in the list it returns a score in the range [0, 1], which predicts the probability 
     of the sentence to contains claim for or against the topic. 

     """


def show_claim_classifier_example():
    HEADERS = {'content-type': 'application/json',
               'charset': 'UTF-8'}
    uri = base_server_url_prefix + 'wp234-claim-sentence-motion' + base_server_url + 'score/'
    payload = {'pairs': [
        'We should legalize cannabis^^^A cross-party report found good evidence that cannabis treatments can help alleviate the symptoms of chronic pain, multiple sclerosis, nausea and vomiting, particularly in the context of chemotherapy, and anxiety.',
        'We should legalize cannabis^^^cannabis is dangerous']}
    response = requests.post(url=uri, json=payload, headers=HEADERS)
    print(response.json())


def show_term_wikifier_example():
    HEADERS = {'content-type': 'application/json',
               'charset': 'UTF-8'}
    uri = base_server_url_prefix + 'tw' + base_server_url + 'TermWikifier/v2/annotate'
    payload = {
        'config': '',
        'contextTitles': [],
        'contextText': '',
        'textsToAnnotate': [
            {'text': 'Cats are much better than dogs',
             'contextTitles': []},
            {'text': 'Give me a wiki to stand, and I shall move the world.',
             'contextTitles': []},
        ]
    }
    response = requests.post(url=uri, json=payload, headers=HEADERS)
    print(response.json())


"""
    """
def show_term_relater_example():
    HEADERS = {'content-type': 'application/json',
               'charset': 'UTF-8'}
    uri = base_server_url_prefix + 'term-relater-service' + base_server_url + 'score/'
    payload = {'pairs':
        [
            {'first': 'Soldier',
             'second': 'Loneliness'},
            {'first': 'Army',
             'second': 'Soldier'}
        ]
    }
    response = requests.post(url=uri, json=payload, headers=HEADERS)
    print(response.json())

    """
    """
def show_speech_construction_example():
    HEADERS = {
    'Accept-Encoding': 'gzip, deflate',
    'Content-Type': 'application/json',
    'charset': 'utf-8'}
    uri = base_server_url_prefix + 'speech-construction' + base_server_url + '/api/v1/speech/generate'
    motionGenerationId = '1187'
    motionText = "We should legalize cannabis"
    creationTime = "Sun Sep 29 09:38:48 IDT 2019"
    polarity = "pro"
    dominantConcept = "Cannabis"
    eventId = "eventId-e2e"
    publish = "false"
    assess = "false"
    customizations = []
    payload = {'motionGenerationId' : motionGenerationId,
               'motionText' : motionText,
               'creationTime' : creationTime,
               'arguments': sc_example.args,
               'polarity' : polarity,
               'dominantConcept' : dominantConcept,
               'eventId' : eventId,
               'publish' : publish,
               'assess' : assess,
               'customizations' : customizations

               }
    response = requests.post(url=uri, json=payload, headers=HEADERS)
    print(response.json())


if __name__ == '__main__':

    print('\nClustering example:')
    show_clustering_example()

    print('\nQuality example:')
    show_quality_example()

    print('\nProCon example:')
    show_pro_con_example()

    print('\nTerm wikifier example:')
    show_term_wikifier_example()

    print('\nEvidence classifier example:')
    show_evidence_classifier_example()

    print('\nClaim classifier example:')
    show_claim_classifier_example()

    print('\nTerm relater example:')
    show_term_relater_example()

    print('\nSpeech construction example:')
    show_speech_construction_example()





from debater_python_api.api.clients.key_point_analysis.KpAnalysisUtils import KpAnalysisUtils

if __name__ == '__main__':
    KpAnalysisUtils.init_logger()

    file = '/Users/yoavkantor/Documents/projects/keypoint-matching-kps-graphs-ui/static/data/default_results.json'
    KpAnalysisUtils.graph_data_to_hierarchical(file)


# (C) Copyright IBM Corp. 2020.

from debater_python_api.api.debater_api import DebaterApi
from debater_python_api.api.sentence_level_index.client.sentence_query_base import SimpleQuery
from debater_python_api.api.sentence_level_index.client.sentence_query_request import SentenceQueryRequest
from debater_python_api.api.sentence_level_index.client.article_retrieval_request import ArticleRetrievalRequest

debater_api = DebaterApi('PUT_YOUR_API_KEY_HERE')

index_searcher_client = debater_api.get_index_searcher_client()

query = SimpleQuery(is_ordered=False, window_size=10)
query.add_concept_element(['Wind power', 'Wind farm'])
query_request = SentenceQueryRequest(query=query.get_sentence_query(), size=200, sentenceLength=(7, 60))

sentences = index_searcher_client.run(query_request, return_full_record=True)

for sentence in sentences:
    print(sentence)


print('\n\nprev sentences')
query_request = ArticleRetrievalRequest(articleIds=[sentence.parentArticleId for sentence in sentences])
articles = index_searcher_client.get_articles(query_request)
for article, sentence in zip(articles, sentences):
    prev_sentence_position = sentence.sentencePosition - 1
    if 0 <= prev_sentence_position < len(article['sentencesFields']):
        print(article['articleFields']['fullText'][int(article['sentencesFields'][prev_sentence_position]['spanStart']):
                                                   int(article['sentencesFields'][prev_sentence_position]['spanEnd'])])


# (C) Copyright IBM Corp. 2020.
import datetime
import logging
from debater_python_api.api.clients.abstract_client import AbstractClient

endpoint = '/api/v1/theme/extract/'

class ThemeExtractionClient(AbstractClient):
    def __init__(self, apikey):
        AbstractClient.__init__(self, apikey)
        self.host = 'https://theme-extraction.debater.res.ibm.com'

    def run(self, topic, dominant_concept, clusters, themes_to_exclude=[]):
        time_stamp_start = datetime.datetime.now().timestamp()
        if (len(topic) == 0 or len(dominant_concept) == 0):
            raise RuntimeError('empty input argument: topic "{}" or dominant_concept "{}".'.format(topic, dominant_concept))

        payload = {
            'elements': clusters,
            'dominantConcept': dominant_concept,
            'topic': topic,
            'themesToExclude': themes_to_exclude
        }
        json = self.do_run(payload, endpoint=endpoint)
        theme_extraction_result = []
        results = json['elements']
        for i in range(len(results)):
            cluster_themes = results[i]['themes']
            themes = [{'theme' : cluster_themes[j]['theme'], 'pValue' : cluster_themes[j]['p_value']}
                      for j in range(len(cluster_themes)) if cluster_themes[j]['is_valid'] is True]
            theme_extraction_result.append(themes)

        time_stamp_end = datetime.datetime.now().timestamp()
        logging.info('theme_extraction_client.run = {}ms.'.format(1000*(time_stamp_end - time_stamp_start)))

        return theme_extraction_result


    def run_and_get_all_themes_supporters(self, topic, dominant_concept, clusters, themes_to_exclude=[], p_value_th = None):
        time_stamp_start = datetime.datetime.now().timestamp()
        if (len(topic) == 0 or len(dominant_concept) == 0):
            raise RuntimeError('empty input argument: topic "{}" or dominant_concept "{}".'.format(topic, dominant_concept))

        payload = {
            'elements': clusters,
            'dominantConcept': dominant_concept,
            'topic': topic,
            'themesToExclude': themes_to_exclude
        }
        json = self.do_run(payload)
        theme_extraction_result = []
        results = json['elements']
        for i in range(len(results)):
            cluster_themes = results[i]['themes']
            filtered_cluster_themes = [theme for theme in cluster_themes if
                                       (p_value_th is not None and theme['p_value'] <= p_value_th)
                                       or
                                       (theme['is_valid'] is True)]

            themes = [{'theme' : cluster_themes[j]['theme'],
                       'p_value' : cluster_themes[j]['p_value'],
                       'num_apearances': cluster_themes[j]['term_frequency_in_cluster'],
                       'num_supporters' : cluster_themes[j]['num_of_supporters'],
                       'indices_of_supporters' : cluster_themes[j]['indices_of_supporters'][0:cluster_themes[j]['num_of_supporters']]
                       }
                      for j in range(len(filtered_cluster_themes))]
            theme_extraction_result.append(themes)

        time_stamp_end = datetime.datetime.now().timestamp()
        logging.info('theme_extraction_client.run = {}ms.'.format(1000*(time_stamp_end - time_stamp_start)))
        return theme_extraction_result




# (C) Copyright IBM Corp. 2020.

from debater_python_api.api.debater_api import DebaterApi

debater_api = DebaterApi('PUT_YOUR_API_KEY_HERE')
claim_detection_client = debater_api.get_claim_detection_client()

topic = 'We should legalize cannabis'
sentences = ['A cross-party report found good evidence that cannabis treatments can help alleviate the symptoms of chronic pain, multiple sclerosis, nausea and vomiting, particularly in the context of chemotherapy, and anxiety.',
             'A recent federal study indicates that cannabis is dangerous',
             'cannabis is dangerous',
             'The apple tree is green']

sentence_topic_dicts = [{'sentence' : sentence, 'topic' : topic } for sentence in sentences]

scores = claim_detection_client.run(sentence_topic_dicts)

for j in range(len(sentence_topic_dicts)):
    print('topic: '+sentence_topic_dicts[j]['topic'])
    print('sentence: '+sentence_topic_dicts[j]['sentence'])
    print('score: '+"{:.4f}".format(scores[j]))
    print()



import json
import logging
import os
from collections import defaultdict
from pathlib import Path
import pandas as pd
import numpy as np

from debater_python_api.api.clients.key_point_analysis.KpaExceptions import KpaIllegalInputException
from debater_python_api.utils.kp_analysis_utils import create_dict_to_list, argmax


class KpAnalysisUtils:
    '''
    A class with static methods for utilities that assist with the key point analysis service.
    '''

    @staticmethod
    def print_result(result, print_matches=False):
        '''
        Prints the key point analysis result to console.
        :param result: the result, returned by method get_result in KpAnalysisTaskFuture.
        '''
        def print_kp(kp, n_matches, n_matches_subtree, depth, print_matches, keypoint_matching):
            spaces = '     ' * depth
            has_n_matches_subtree = n_matches_subtree is not None
            logging.info('%s    %d%s - %s' % (spaces, n_matches_subtree if has_n_matches_subtree else n_matches,
                                       (' - %d' % n_matches) if has_n_matches_subtree else '', kp))
            if print_matches:
                for match in keypoint_matching['matching']:
                    logging.info('%s    %s - %s' % (spaces + '    ', str(match['score']), match['sentence_text']))

        kp_to_n_matches_subtree = defaultdict(int)
        parents = list()
        parent_to_kids = defaultdict(list)
        for keypoint_matching in result['keypoint_matchings']:
            kp = keypoint_matching['keypoint']
            kp_to_n_matches_subtree[kp] += len(keypoint_matching['matching'])
            parent = keypoint_matching.get("parent", None)
            if parent is None or parent == 'root':
                parents.append(keypoint_matching)
            else:
                parent_to_kids[parent].append(keypoint_matching)
                kp_to_n_matches_subtree[parent] += len(keypoint_matching['matching'])

        parents.sort(key=lambda x: kp_to_n_matches_subtree[x['keypoint']], reverse=True)

        logging.info('Result:')
        for parent in parents:
            kp = parent['keypoint']
            print_kp(kp, len(parent['matching']), None if len(parent_to_kids[kp]) == 0 else kp_to_n_matches_subtree[kp], 0, print_matches, parent)
            for kid in parent_to_kids[kp]:
                kid_kp = kid['keypoint']
                print_kp(kid_kp, len(kid['matching']), None, 1, print_matches, kid)

    @staticmethod
    def print_report(user_report):
        '''
        Prints the user_report to console.
        :param user_report: the user report, returned by method get_full_report in KpAnalysisClient.
        '''
        logging.info('User Report:')
        comments_statuses = user_report['domains_status']
        logging.info('  Comments status by domain (%d domains):' % len(comments_statuses))
        if len(comments_statuses) == 0:
            logging.info('    User has no domains')
        else:
            for status in comments_statuses:
                logging.info(f'    Domain: {status["domain"]}, Domain Params: {status["domain_params"]}, Status: {status["data_status"]}')
        kp_analysis_statuses = user_report['kp_analysis_status']
        logging.info(f'  Key point analysis - jobs status ({len(kp_analysis_statuses)} jobs):')
        if len(kp_analysis_statuses) == 0:
            logging.info('    User has no key point analysis jobs history')
        else:
            for kp_analysis_status in kp_analysis_statuses:
                logging.info(f'    Job: {str(kp_analysis_status)}')

    @staticmethod
    def set_stance_to_result(result, stance):
        for keypoint_matching in result['keypoint_matchings']:
            keypoint_matching['stance'] = stance
        return result

    @staticmethod
    def merge_two_results(result1, result2):
        result = {'keypoint_matchings': result1['keypoint_matchings'] + result2['keypoint_matchings']}
        result['keypoint_matchings'].sort(key=lambda matchings: len(matchings['matching']), reverse=True)
        return result

    @staticmethod
    def write_result_to_csv(result, result_file):
        '''
        Writes the key point analysis result to file.
        Creates two files:
        * matches file: a file with all sentence-key point matches, saved in result_file path.
        * summary file: a summary file with all key points and their aggregated information, saved in result_file path with suffix: _kps_summary.csv.
        :param result: the result, returned by method get_result in KpAnalysisTaskFuture.
        :param result_file: a path to the file that will be created (should have a .csv suffix, otherwise it will be added).
        '''
        def _write_df_to_file(df, file):
            logging.info("Writing dataframe to: " + file)
            file_path = Path(file)
            if not os.path.exists(file_path.parent):
                logging.info('creating directory: %s' % str(file_path.parent))
                os.makedirs(file_path.parent)
            df.to_csv(file, index=False)

        if 'keypoint_matchings' not in result:
            logging.info("No keypoint matchings results")
            return

        if '.csv' not in result_file:
            result_file += '.csv'

        all_sentneces_ids = set([m['comment_id'] + "_" + str(m['sentence_id'])
                                 for keypoint_matching in result['keypoint_matchings'] for m in
                                 keypoint_matching['matching']])
        total_sentences = len(all_sentneces_ids)

        all_comment_ids = set([m["comment_id"] for keypoint_matching in result['keypoint_matchings'] for m in keypoint_matching['matching']])
        total_comments = len(all_comment_ids)
        all_mapped_comment_ids = set([m["comment_id"] for keypoint_matching in result['keypoint_matchings'] if
                                      keypoint_matching['keypoint'] != 'none' for m in keypoint_matching['matching']])

        total_unmapped_comments = total_comments - len(all_mapped_comment_ids)

        summary_rows = []
        matchings_rows = []
        kp_to_parent = {}
        kps_have_stance = False
        sentences_have_stance = False
        stance_keys = []
        for keypoint_matching in result['keypoint_matchings']:
            kp = keypoint_matching['keypoint']
            kp_stance = keypoint_matching.get('stance', None)
            n_sentences = len(keypoint_matching['matching'])
            sentence_coverage = n_sentences / total_sentences if total_sentences > 0 else 0.0
            if kp == "none":
                n_comments = total_unmapped_comments
            else:
                n_comments = len(set([m["comment_id"] for m in keypoint_matching['matching']]))
            comments_coverage = n_comments / total_comments if total_comments > 0 else 0.0

            summary_row = [kp, n_sentences, sentence_coverage, n_comments, comments_coverage]
            if kp_stance is not None:
                kps_have_stance = True
            if kps_have_stance:
                summary_row.append(kp_stance if kp_stance else "")

            kp_to_parent[kp] = keypoint_matching.get("parent", 'root')
            kp_scores = [0, 0, 0]
            for match in keypoint_matching['matching']:
                match_row = [kp, match["sentence_text"], match["score"], match["comment_id"], match["sentence_id"],
                             match["sents_in_comment"], match["span_start"], match["span_end"], match["num_tokens"],
                             match["argument_quality"],  match.get("kp_quality", 0)]

                if match["sentence_text"] == kp:
                    kp_q = match.get("kp_quality", 0)
                    kp_scores = [match["num_tokens"], match["argument_quality"], kp_q]

                if 'stance' in match:
                    stance_dict = match['stance']
                    stance_tups = list(stance_dict.items())
                    stance_keys = [t[0] for t in stance_tups]
                    stance_scores = [t[1] for t in stance_tups]
                    match_row.extend(stance_scores)
                    stance_conf = np.max(stance_scores)
                    selected_stance = stance_keys[np.argmax(stance_scores)]
                    match_row.extend([selected_stance, stance_conf])
                    sentences_have_stance = True
                if kp_stance is not None:
                    match_row.append(kp_stance)
                matchings_rows.append(match_row)

            summary_row.extend(kp_scores)
            summary_rows.append(summary_row)

        summary_rows = sorted(summary_rows, key=lambda x: x[1], reverse=True)
        summary_cols = ["kp", "#sentences", 'sentences_coverage', '#comments', 'comments_coverage']
        if kps_have_stance:
            summary_cols.append('stance')
        summary_cols.extend(["num_tokens", "argument_quality", "kp_quality"])

        summary_df = pd.DataFrame(summary_rows, columns=summary_cols)

        if len(set(kp_to_parent.values())) > 1:
            summary_df.loc[:, "parent"] = summary_df.apply(lambda x: kp_to_parent[x["kp"]], axis=1)
            parent_to_kps = {p: list(filter(lambda x: kp_to_parent[x] == p, kp_to_parent.keys()))
                             for p in set(kp_to_parent.values())}
            parent_to_kps.update({p: [] for p in set(parent_to_kps["root"]).difference(parent_to_kps.keys())})
            kp_to_n_args = dict(zip(summary_df["kp"], summary_df["#sentences"]))
            kp_to_n_args_sub = {kp: np.sum([kp_to_n_args[c_kp] for c_kp in set(parent_to_kps.get(kp, []) + [kp])])
                                for kp in kp_to_parent}
            kp_to_n_args_sub["root"] = np.sum(list(summary_df["#sentences"]))
            summary_df.loc[:, "#sents_in_subtree"] = summary_df.apply(lambda x: kp_to_n_args_sub[x["kp"]], axis=1)

            hierarchy_data = [[p, len(parent_to_kps[p]), kp_to_n_args_sub[p], parent_to_kps[p]] for p in parent_to_kps]
            hierarchy_df = pd.DataFrame(hierarchy_data,
                                        columns=["top_kp", "#level_2_kps", "#sents_in_subtree", "level_2_kps"])
            hierarchy_df.sort_values(by=["#sents_in_subtree"], ascending=False, inplace=True)

            hierarchy_file = result_file.replace(".csv", "_kp_hierarchy.csv")
            _write_df_to_file(hierarchy_df, hierarchy_file)

        summary_file = result_file.replace(".csv", "_kps_summary.csv")
        _write_df_to_file(summary_df, summary_file)

        matchings_cols = ["kp", "sentence_text", "match_score", 'comment_id', 'sentence_id', 'sents_in_comment', 'span_start',
                'span_end', 'num_tokens', 'argument_quality', 'kp_quality']
        if sentences_have_stance:
            matchings_cols.extend([f'{k}_score' for k in stance_keys] + ["selected_stance", "stance_conf"])
        if kps_have_stance:
            matchings_cols.append('kp_stance')
        match_df = pd.DataFrame(matchings_rows, columns=matchings_cols)
        _write_df_to_file(match_df, result_file)

    @staticmethod
    def write_sentences_to_csv(sentences, out_file):
        def get_selected_stance(r):
            stance_dict = dict(r["stance_dict"])
            stance_list = list(stance_dict.items())
            stance_list = sorted(stance_list, reverse=True, key=lambda item: item[1])
            stance, conf = stance_list[0]
            r["stance"] = stance
            r["stance_conf"] = conf
            return r

        if len(sentences) == 0:
            logging.info('there are no sentences, not saving file')
            return

        cols = list(sentences[0].keys())
        rows = [[s[col] for col in cols] for s in sentences]
        df = pd.DataFrame(rows, columns=cols)
        if "stance_dict" in cols:
            df = df.apply(lambda r: get_selected_stance(r), axis=1)
        df.to_csv(out_file, index=False)

    @staticmethod
    def init_logger():
        '''
        Inits the logger for more informative console prints.
        '''
        from logging import getLogger, getLevelName, Formatter, StreamHandler
        log = getLogger()
        log.setLevel(getLevelName('INFO'))
        log_formatter = Formatter("%(asctime)s [%(levelname)s] %(filename)s %(lineno)d: %(message)s")

        console_handler = StreamHandler()
        console_handler.setFormatter(log_formatter)
        log.handlers = []
        log.addHandler(console_handler)

    @staticmethod
    def create_domain_ignore_exists(client, domain, domain_params):
        try:
            client.create_domain(domain, domain_params)
            logging.info(f'domain: {domain} was created')
        except KpaIllegalInputException as e:
            if 'already exist' not in str(e):
                raise e
            logging.info(f'domain: {domain} already exists, domain_params are NOT updated.')

    @staticmethod
    def delete_domain_ignore_doesnt_exist(client, domain):
        try:
            client.delete_domain_cannot_be_undone(domain)
            logging.info(f'domain: {domain} was deleted')
        except KpaIllegalInputException as e:
            if 'doesn\'t have domain' not in str(e):
                raise e
            logging.info(f'domain: {domain} doesn\'t exist.')

    @staticmethod
    def create_graph_data_file_for_ui(result_file, min_n_similar_matches=5, n_matches_samples=20):
        '''
        translates the result file (full result, not the summary) into a json that is loadable in the kpa-key-points-graph-ui
        :param result_file: full results file (with sentence to keypoint mappings).
        :param min_n_similar_matches: minimal number of sentences in the key points intersection for creating an edge (relation between the key points).
        :return: creates a new json file (located near result_file with a new suffix).
        '''
        def graph_to_graph_data(graph, n_sentences):
            for node in graph['nodes']:
                node['relative_val'] = float(node['n_matches']) / float(n_sentences)

            graph_data = []
            for n in graph['nodes']:
                id = n['id']
                graph_data.append({'type': 'node',
                                   'data':
                                       {
                                           'id': str(id),
                                           'kp': str(n["kp"]),
                                           'n_matches': str(n["n_matches"]),
                                           'n_sentences': str(n_sentences),
                                           'relative_val': round(n['relative_val'], 3),
                                           'matches': n['matches']
                                       }})

            for i, e in enumerate(graph['edges']):
                graph_data.append(
                    {'type': 'edge',
                     'data':
                         {'id': f'e{i}',
                          'source': str(e['source']),
                          'target': str(e['target']),
                          'score': round(e['score'], 2)
                          }})
            return graph_data

        def create_dict_to_list(list_tups):
            res = defaultdict(list)
            for x, y in list_tups:
                res[x].append(y)
            return dict(res)

        def read_tups_from_csv(filename):
            logging.info(f'reading file: {filename}')
            input_df = pd.read_csv(filename)
            cols = list(input_df.columns)
            tups = list(input_df.to_records(index=False))
            tups = [tuple([str(v) if str(v) != 'nan' else '' for v in t]) for t in tups]
            return tups, cols

        def read_dicts_from_csv(filename):
            tups, cols = read_tups_from_csv(filename)
            dicts = [{} for _ in tups]
            dicts_tups = list(zip(dicts, tups))
            for i, c in enumerate(cols):
                for d, t in dicts_tups:
                    d[c] = t[i]
            return dicts, cols

        def get_node(i, kp, kp_to_dicts, n_matches_samples):
            return {'id': i,
                    'kp': kp,
                    'n_matches': len(kp_to_dicts[kp]),
                    'matches': [{'sentence_text': d["sentence_text"],
                                 'match_score': float(d["match_score"])}
                                for d in kp_to_dicts[kp][:n_matches_samples]]
                    }

        logging.info(f'Creating key points graph data-file for results file: {result_file}')
        dicts, _ = read_dicts_from_csv(result_file)

        n_sentences = len(set([d['sentence_text'] for d in dicts]))

        kp_to_dicts = create_dict_to_list([(d['kp'], d) for d in dicts])
        all_kps = set(kp_to_dicts.keys())
        all_kps = all_kps - {'none'}
        all_kps = sorted(list(all_kps), key=lambda kp: len(kp_to_dicts[kp]), reverse=True)
        kp_to_sentences = {kp: set([(d['comment_id'], d['sentence_id']) for d in kp_dicts]) for kp, kp_dicts in
                           kp_to_dicts.items()}

        # edge (i, j): what percentage of sentences in i are also sentences in j
        edge_to_score = {}
        for i in range(len(all_kps)):
            for j in range(len(all_kps)):
                if i == j:
                    continue
                else:
                    kp_i = all_kps[i]
                    kp_j = all_kps[j]
                    sents_i = kp_to_sentences[kp_i]
                    sents_j = kp_to_sentences[kp_j]
                    n_sentences_i_in_j = len([s for s in sents_i if s in sents_j])
                    if n_sentences_i_in_j > min_n_similar_matches:
                        edge_to_score[(i, j)] = n_sentences_i_in_j / len(sents_i)
        nodes = [get_node(i, kp, kp_to_dicts, n_matches_samples) for i, kp in enumerate(all_kps)]
        edges = [{'source': i, 'target': j, 'score': edge_to_score[i, j]} for (i, j) in edge_to_score]
        graph = {'nodes': nodes, 'edges': edges}

        graph_data = graph_to_graph_data(graph, n_sentences)

        out_file = result_file.replace('.csv', '_graph_data.json')
        logging.info(f'saving graph in file: {out_file}')
        logging.info(f'saving graph in file: {out_file}')
        with open(out_file, 'w') as f:
            json.dump(graph_data, f)

    @staticmethod
    def graph_data_to_hierarchical(result_graph_data_json,
                                   filter_small_nodes=-1,
                                   filter_min_relations=-1,
                                   filter_big_to_small_edges=True,
                                   save_hierarchical_textual_bullets=True,
                                   save_hierarchical_graph_data=True):
        def get_hierarchical_bullets_aux(id_to_kids, id_to_node, id, tab, res):
            msg = f'{"  " * tab} * {id_to_node[id]["data"]["kp"]} ({id_to_node[id]["data"]["n_matches"]} matches)'
            res.append(msg)
            if id in id_to_kids:
                kids = id_to_kids[id]
                kids = sorted(kids, key=lambda n: int(id_to_node[n]['data']['n_matches']), reverse=True)
                for k in kids:
                    get_hierarchical_bullets_aux(id_to_kids, id_to_node, k, tab + 1, res)

        def get_hierarchical_bullets(roots, id_to_kids, id_to_node):
            res = []
            tab = 0
            roots = sorted(roots, key=lambda n: int(id_to_node[n]['data']['n_matches']), reverse=True)
            for root in roots:
                get_hierarchical_bullets_aux(id_to_kids, id_to_node, root, tab, res)
            return res

        print(f'creating hierarchical graph from file: {result_graph_data_json}')
        data = json.load(open(result_graph_data_json))
        nodes = [d for d in data if d['type'] == 'node']
        edges = [d for d in data if d['type'] == 'edge']

        if filter_small_nodes > 0:
            nodes = [n for n in nodes if
                     (round(float(n['data']['n_matches']) / float(n['data']['n_sentences']), 3) >= filter_small_nodes)]
        nodes_ids = [n['data']['id'] for n in nodes]
        id_to_node = {d['data']['id']: d for d in nodes}

        if filter_min_relations > 0:
            edges = [e for e in edges if float(e['data']['score']) >= filter_min_relations]

        edges = [e for e in edges if (e['data']['source'] in nodes_ids and e['data']['target'] in nodes_ids)]

        if filter_big_to_small_edges:
            edges = [e for e in edges if int(id_to_node[e['data']['source']]['data']['n_matches']) <= int(
                id_to_node[e['data']['target']]['data']['n_matches'])]

        source_target_to_score = {(e['data']['source'], e['data']['target']): e['data']['score'] for e in edges}

        # keep only edges to max parent
        id_to_parents = create_dict_to_list([(e['data']['source'], e['data']['target']) for e in edges])
        id_to_max_parent = {}
        for id, parents in id_to_parents.items():
            scores = [source_target_to_score[(id, parent)] if (id, parent) in source_target_to_score else 0.0 for parent in parents]
            max_score_idx = argmax(scores)
            max_parent = parents[max_score_idx]
            id_to_max_parent[id] = max_parent

        edges = [e for e in edges if e['data']['source'] in id_to_max_parent and id_to_max_parent[e['data']['source']] == e['data']['target']]

        if save_hierarchical_textual_bullets:
            id_to_kids = create_dict_to_list([(e['data']['target'], e['data']['source']) for e in edges])
            all_kids = set()
            for id, kids in id_to_kids.items():
                all_kids = all_kids.union(kids)

            root_ids = set(nodes_ids).difference(all_kids)
            res = get_hierarchical_bullets(root_ids, id_to_kids, id_to_node)
            if '_graph_data.json' in result_graph_data_json:
                out_file = result_graph_data_json.replace('_graph_data.json', '_hierarchical.txt')
            else:
                out_file = result_graph_data_json.replace('.json', '_hierarchical.txt')
            logging.info(f'saving graph in file: {out_file}')
            with open(out_file, 'w') as f:
                for line in res:
                    f.write("%s\n" % line)

        if save_hierarchical_graph_data:
            graph_data = nodes + edges
            if '_graph_data.json' in result_graph_data_json:
                out_file = result_graph_data_json.replace('_graph_data.json', '_hierarchical_graph_data.json')
            else:
                out_file = result_graph_data_json.replace('.json', '_hierarchical_graph_data.json')
            logging.info(f'saving graph in file: {out_file}')
            with open(out_file, 'w') as f:
                json.dump(graph_data, f)

